"""
Template tags for Django
"""
# TODO: add in Jinja tags if Coffin is available

from django import template
from django.contrib.contenttypes.models import ContentType
from django.db.models import ObjectDoesNotExist

from djangoratings.models import Vote

register = template.Library()

class RatingByRequestNode(template.Node):
    def __init__(self, request, obj, context_var):
        self.request = request
        self.obj, self.field_name = obj.split('.')
        self.context_var = context_var
    
    def render(self, context):
        try:
            request = template.resolve_variable(self.request, context)
            obj = template.resolve_variable(self.obj, context)
            field = getattr(obj, self.field_name)
        except (template.VariableDoesNotExist, AttributeError):
            return ''
        try:
            vote = field.get_rating_for_user(request.user, request.META['REMOTE_ADDR'], request.COOKIES)
            context[self.context_var] = vote
        except ObjectDoesNotExist:
            context[self.context_var] = 0
        return ''

def do_rating_by_request(parser, token):
    """
    Retrieves the ``Vote`` cast by a user on a particular object and
    stores it in a context variable. If the user has not voted, the
    context variable will be 0.
    
    Example usage::
    
        {% rating_by_request request on instance as vote %}
    """
    
    bits = token.contents.split()
    if len(bits) != 6:
        raise template.TemplateSyntaxError("'%s' tag takes exactly five arguments" % bits[0])
    if bits[2] != 'on':
        raise template.TemplateSyntaxError("second argument to '%s' tag must be 'on'" % bits[0])
    if bits[4] != 'as':
        raise template.TemplateSyntaxError("fourth argument to '%s' tag must be 'as'" % bits[0])
    return RatingByRequestNode(bits[1], bits[3], bits[5])
register.tag('rating_by_request', do_rating_by_request)

class RatingByUserNode(RatingByRequestNode):
    def render(self, context):
        try:
            user = template.resolve_variable(self.request, context)
            obj = template.resolve_variable(self.obj, context)
            field = getattr(obj, self.field_name)
        except template.VariableDoesNotExist:
            return ''
        try:
            vote = field.get_rating_for_user(user)
            context[self.context_var] = vote
        except ObjectDoesNotExist:
            context[self.context_var] = 0
        return ''

def do_rating_by_user(parser, token):
    """
    Retrieves the ``Vote`` cast by a user on a particular object and
    stores it in a context variable. If the user has not voted, the
    context variable will be 0.
    
    Example usage::
    
        {% rating_by_user user on instance as vote %}
    """
    
    bits = token.contents.split()
    if len(bits) != 6:
        raise template.TemplateSyntaxError("'%s' tag takes exactly five arguments" % bits[0])
    if bits[2] != 'on':
        raise template.TemplateSyntaxError("second argument to '%s' tag must be 'on'" % bits[0])
    if bits[4] != 'as':
        raise template.TemplateSyntaxError("fourth argument to '%s' tag must be 'as'" % bits[0])
    return RatingByUserNode(bits[1], bits[3], bits[5])
register.tag('rating_by_user', do_rating_by_user)



import unittest
import random

from django.db import models
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.conf import settings

from exceptions import *
from models import Vote, SimilarUser, IgnoredObject
from fields import AnonymousRatingField, RatingField

settings.RATINGS_VOTES_PER_IP = 1

class RatingTestModel(models.Model):
    rating = AnonymousRatingField(range=2, can_change_vote=True)
    rating2 = RatingField(range=2, can_change_vote=False)
    
    def __unicode__(self):
        return unicode(self.pk)

class RatingTestCase(unittest.TestCase):
    def testRatings(self):
        instance = RatingTestModel.objects.create()
        
        # Test adding votes
        instance.rating.add(score=1, user=None, ip_address='127.0.0.1')
        self.assertEquals(instance.rating.score, 1)
        self.assertEquals(instance.rating.votes, 1)

        # Test adding votes
        instance.rating.add(score=2, user=None, ip_address='127.0.0.2')
        self.assertEquals(instance.rating.score, 3)
        self.assertEquals(instance.rating.votes, 2)

        # Test changing of votes
        instance.rating.add(score=2, user=None, ip_address='127.0.0.1')
        self.assertEquals(instance.rating.score, 4)
        self.assertEquals(instance.rating.votes, 2)
        
        # Test users
        user = User.objects.create(username=str(random.randint(0, 100000000)))
        user2 = User.objects.create(username=str(random.randint(0, 100000000)))
        
        instance.rating.add(score=2, user=user, ip_address='127.0.0.3')
        self.assertEquals(instance.rating.score, 6)
        self.assertEquals(instance.rating.votes, 3)
        
        instance.rating2.add(score=2, user=user, ip_address='127.0.0.3')
        self.assertEquals(instance.rating2.score, 2)
        self.assertEquals(instance.rating2.votes, 1)
        
        self.assertRaises(IPLimitReached, instance.rating2.add, score=2, user=user2, ip_address='127.0.0.3')

        # Test deletion hooks
        Vote.objects.filter(ip_address='127.0.0.3').delete()
        
        instance = RatingTestModel.objects.get(pk=instance.pk)

        self.assertEquals(instance.rating.score, 4)
        self.assertEquals(instance.rating.votes, 2)
        self.assertEquals(instance.rating2.score, 0)
        self.assertEquals(instance.rating2.votes, 0)

class RecommendationsTestCase(unittest.TestCase):
    def setUp(self):
        self.instance = RatingTestModel.objects.create()
        self.instance2 = RatingTestModel.objects.create()
        self.instance3 = RatingTestModel.objects.create()
        self.instance4 = RatingTestModel.objects.create()
        self.instance5 = RatingTestModel.objects.create()
        
        # Test users
        self.user = User.objects.create(username=str(random.randint(0, 100000000)))
        self.user2 = User.objects.create(username=str(random.randint(0, 100000000)))
    
    def testExclusions(self):
        Vote.objects.all().delete()

        self.instance.rating.add(score=1, user=self.user, ip_address='127.0.0.1')
        self.instance2.rating.add(score=1, user=self.user, ip_address='127.0.0.1')
        self.instance3.rating.add(score=1, user=self.user, ip_address='127.0.0.1')
        self.instance4.rating.add(score=1, user=self.user, ip_address='127.0.0.1')
        self.instance5.rating.add(score=1, user=self.user, ip_address='127.0.0.1')
        self.instance.rating.add(score=1, user=self.user2, ip_address='127.0.0.2')

        # we should only need to call this once
        SimilarUser.objects.update_recommendations()

        self.assertEquals(SimilarUser.objects.count(), 2)

        recs = list(SimilarUser.objects.get_recommendations(self.user2, RatingTestModel))
        self.assertEquals(len(recs), 4)
        
        ct = ContentType.objects.get_for_model(RatingTestModel)
        
        IgnoredObject.objects.create(user=self.user2, content_type=ct, object_id=self.instance2.pk)

        recs = list(SimilarUser.objects.get_recommendations(self.user2, RatingTestModel))
        self.assertEquals(len(recs), 3)

        IgnoredObject.objects.create(user=self.user2, content_type=ct, object_id=self.instance3.pk)
        IgnoredObject.objects.create(user=self.user2, content_type=ct, object_id=self.instance4.pk)

        recs = list(SimilarUser.objects.get_recommendations(self.user2, RatingTestModel))
        self.assertEquals(len(recs), 1)
        self.assertEquals(recs, [self.instance5])
        
        self.instance5.rating.add(score=1, user=self.user2, ip_address='127.0.0.2')
        recs = list(SimilarUser.objects.get_recommendations(self.user2, RatingTestModel))
        self.assertEquals(len(recs), 0)
    
    def testSimilarUsers(self):
        Vote.objects.all().delete()

        self.instance.rating.add(score=1, user=self.user, ip_address='127.0.0.1')
        self.instance2.rating.add(score=1, user=self.user, ip_address='127.0.0.1')
        self.instance3.rating.add(score=1, user=self.user, ip_address='127.0.0.1')
        self.instance4.rating.add(score=1, user=self.user, ip_address='127.0.0.1')
        self.instance5.rating.add(score=1, user=self.user, ip_address='127.0.0.1')
        self.instance.rating.add(score=1, user=self.user2, ip_address='127.0.0.2')
        self.instance2.rating.add(score=1, user=self.user2, ip_address='127.0.0.2')
        self.instance3.rating.add(score=1, user=self.user2, ip_address='127.0.0.2')
        
        SimilarUser.objects.update_recommendations()

        self.assertEquals(SimilarUser.objects.count(), 2)

        recs = list(SimilarUser.objects.get_recommendations(self.user2, RatingTestModel))
        self.assertEquals(len(recs), 2)
        
        self.instance4.rating.add(score=1, user=self.user2, ip_address='127.0.0.2')

        SimilarUser.objects.update_recommendations()

        self.assertEquals(SimilarUser.objects.count(), 2)

        recs = list(SimilarUser.objects.get_recommendations(self.user2, RatingTestModel))
        self.assertEquals(len(recs), 1)
        self.assertEquals(recs, [self.instance5])
        
        self.instance5.rating.add(score=1, user=self.user2, ip_address='127.0.0.2')

        SimilarUser.objects.update_recommendations()

        self.assertEquals(SimilarUser.objects.count(), 2)

        recs = list(SimilarUser.objects.get_recommendations(self.user2, RatingTestModel))
        self.assertEquals(len(recs), 0)


from django.core.management.base import NoArgsCommand, CommandError

from djangoratings.models import SimilarUser

class Command(NoArgsCommand):
    def handle_noargs(self, **options):
        SimilarUser.objects.update_recommendations()


from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse, Http404

from exceptions import *
from django.conf import settings
from default_settings import RATINGS_VOTES_PER_IP

class AddRatingView(object):
    def __call__(self, request, content_type_id, object_id, field_name, score):
        """__call__(request, content_type_id, object_id, field_name, score)
        
        Adds a vote to the specified model field."""
        
        try:
            instance = self.get_instance(content_type_id, object_id)
        except ObjectDoesNotExist:
            raise Http404('Object does not exist')
        
        context = self.get_context(request)
        context['instance'] = instance
        
        try:
            field = getattr(instance, field_name)
        except AttributeError:
            return self.invalid_field_response(request, context)
        
        context.update({
            'field': field,
            'score': score,
        })
        
        had_voted = bool(field.get_rating_for_user(request.user, request.META['REMOTE_ADDR'], request.COOKIES))
        
        context['had_voted'] = had_voted
                    
        try:
            adds = field.add(score, request.user, request.META.get('REMOTE_ADDR'), request.COOKIES)
        except IPLimitReached:
            return self.too_many_votes_from_ip_response(request, context)
        except AuthRequired:
            return self.authentication_required_response(request, context)
        except InvalidRating:
            return self.invalid_rating_response(request, context)
        except CannotChangeVote:
            return self.cannot_change_vote_response(request, context)
        except CannotDeleteVote:
            return self.cannot_delete_vote_response(request, context)
        if had_voted:
            return self.rating_changed_response(request, context, adds)
        return self.rating_added_response(request, context, adds)
    
    def get_context(self, request, context={}):
        return context
    
    def render_to_response(self, template, context, request):
        raise NotImplementedError

    def too_many_votes_from_ip_response(self, request, context):
        response = HttpResponse('Too many votes from this IP address for this object.')
        return response

    def rating_changed_response(self, request, context, adds={}):
        response = HttpResponse('Vote changed.')
        if 'cookie' in adds:
            cookie_name, cookie = adds['cookie_name'], adds['cookie']
            if 'deleted' in adds:
                response.delete_cookie(cookie_name)
            else:
                response.set_cookie(cookie_name, cookie, 31536000, path='/') # TODO: move cookie max_age to settings
        return response
    
    def rating_added_response(self, request, context, adds={}):
        response = HttpResponse('Vote recorded.')
        if 'cookie' in adds:
            cookie_name, cookie = adds['cookie_name'], adds['cookie']
            if 'deleted' in adds:
                response.delete_cookie(cookie_name)
            else:
                response.set_cookie(cookie_name, cookie, 31536000, path='/') # TODO: move cookie max_age to settings
        return response

    def authentication_required_response(self, request, context):
        response = HttpResponse('You must be logged in to vote.')
        response.status_code = 403
        return response
    
    def cannot_change_vote_response(self, request, context):
        response = HttpResponse('You have already voted.')
        response.status_code = 403
        return response
    
    def cannot_delete_vote_response(self, request, context):
        response = HttpResponse('You can\'t delete this vote.')
        response.status_code = 403
        return response
    
    def invalid_field_response(self, request, context):
        response = HttpResponse('Invalid field name.')
        response.status_code = 403
        return response
    
    def invalid_rating_response(self, request, context):
        response = HttpResponse('Invalid rating value.')
        response.status_code = 403
        return response
        
    def get_instance(self, content_type_id, object_id):
        return ContentType.objects.get(pk=content_type_id)\
            .get_object_for_this_type(pk=object_id)


class AddRatingFromModel(AddRatingView):
    def __call__(self, request, model, app_label, object_id, field_name, score):
        """__call__(request, model, app_label, object_id, field_name, score)
        
        Adds a vote to the specified model field."""
        try:
            content_type = ContentType.objects.get(model=model, app_label=app_label)
        except ContentType.DoesNotExist:
            raise Http404('Invalid `model` or `app_label`.')
        
        return super(AddRatingFromModel, self).__call__(request, content_type.id,
                                                        object_id, field_name, score)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):
    
    def forwards(self, orm):
        
        # Adding model 'ScoreCorrelation'
        db.create_table('djangoratings_scorecorrelation', (
            ('rank', self.gf('django.db.models.fields.FloatField')()),
            ('to_content_type', self.gf('django.db.models.fields.related.ForeignKey')(related_name='djr_sc_2', to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(related_name='djr_sc_1', to=orm['contenttypes.ContentType'])),
            ('to_object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal('djangoratings', ['ScoreCorrelation'])
    
    
    def backwards(self, orm):
        
        # Deleting model 'ScoreCorrelation'
        db.delete_table('djangoratings_scorecorrelation')
    
    
    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            '_battlenet_profiles': ('django.db.models.fields.TextField', [], {'null': 'True', 'db_column': "'battlenet_profiles'", 'blank': 'True'}),
            'avatar': ('django.db.models.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'bio': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'custom_title': ('django.db.models.fields.CharField', [], {'max_length': '32', 'null': 'True', 'blank': 'True'}),
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'gold': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_moderator': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'level': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'signature': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'}),
            'xp': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'djangoratings.score': {
            'Meta': {'unique_together': "(('content_type', 'object_id', 'key'),)", 'object_name': 'Score'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'mean': ('django.db.models.fields.FloatField', [], {'default': '0.0'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'score': ('django.db.models.fields.IntegerField', [], {}),
            'stddev': ('django.db.models.fields.FloatField', [], {'default': '0.0'}),
            'votes': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        'djangoratings.scorecorrelation': {
            'Meta': {'unique_together': "(('content_type', 'object_id', 'to_content_type', 'to_object_id'),)", 'object_name': 'ScoreCorrelation'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'djr_sc_1'", 'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'rank': ('django.db.models.fields.FloatField', [], {}),
            'to_content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'djr_sc_2'", 'to': "orm['contenttypes.ContentType']"}),
            'to_object_id': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        'djangoratings.vote': {
            'Meta': {'unique_together': "(('content_type', 'object_id', 'key', 'user', 'ip_address'),)", 'object_name': 'Vote'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'votes'", 'to': "orm['contenttypes.ContentType']"}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_changed': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_address': ('django.db.models.fields.IPAddressField', [], {'max_length': '15'}),
            'key': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'score': ('django.db.models.fields.IntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'votes'", 'null': 'True', 'to': "orm['auth.User']"})
        }
    }
    
    complete_apps = ['djangoratings']



#!/usr/bin/env python
import sys

from os.path import dirname, abspath

from django.conf import settings

if not settings.configured:
    settings.configure(
        DATABASE_ENGINE='sqlite3',
        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'djangoratings',
        ]
    )

from django.test.simple import run_tests


def runtests(*test_args):
    if not test_args:
        test_args = ['djangoratings']
    parent = dirname(abspath(__file__))
    sys.path.insert(0, parent)
    failures = run_tests(test_args, verbosity=1, interactive=True)
    sys.exit(failures)


if __name__ == '__main__':
    runtests(*sys.argv[1:])



from south.db import db
from django.db import models
from djangoratings.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'Score'
        db.create_table('djangoratings_score', (
            ('id', orm['djangoratings.Score:id']),
            ('content_type', orm['djangoratings.Score:content_type']),
            ('object_id', orm['djangoratings.Score:object_id']),
            ('key', orm['djangoratings.Score:key']),
            ('score', orm['djangoratings.Score:score']),
            ('votes', orm['djangoratings.Score:votes']),
        ))
        db.send_create_signal('djangoratings', ['Score'])
        
        # Adding model 'Vote'
        db.create_table('djangoratings_vote', (
            ('id', orm['djangoratings.Vote:id']),
            ('content_type', orm['djangoratings.Vote:content_type']),
            ('object_id', orm['djangoratings.Vote:object_id']),
            ('key', orm['djangoratings.Vote:key']),
            ('score', orm['djangoratings.Vote:score']),
            ('user', orm['djangoratings.Vote:user']),
            ('ip_address', orm['djangoratings.Vote:ip_address']),
            ('date_added', orm['djangoratings.Vote:date_added']),
            ('date_changed', orm['djangoratings.Vote:date_changed']),
        ))
        db.send_create_signal('djangoratings', ['Vote'])
        
        # Creating unique_together for [content_type, object_id, key, user, ip_address] on Vote.
        db.create_unique('djangoratings_vote', ['content_type_id', 'object_id', 'key', 'user_id', 'ip_address'])
        
        # Creating unique_together for [content_type, object_id, key] on Score.
        db.create_unique('djangoratings_score', ['content_type_id', 'object_id', 'key'])
        
    
    
    def backwards(self, orm):
        
        # Deleting unique_together for [content_type, object_id, key] on Score.
        db.delete_unique('djangoratings_score', ['content_type_id', 'object_id', 'key'])
        
        # Deleting unique_together for [content_type, object_id, key, user, ip_address] on Vote.
        db.delete_unique('djangoratings_vote', ['content_type_id', 'object_id', 'key', 'user_id', 'ip_address'])
        
        # Deleting model 'Score'
        db.delete_table('djangoratings_score')
        
        # Deleting model 'Vote'
        db.delete_table('djangoratings_vote')
        
    
    
    models = {
        'auth.group': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)"},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '30', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'djangoratings.score': {
            'Meta': {'unique_together': "(('content_type', 'object_id', 'key'),)"},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'score': ('django.db.models.fields.IntegerField', [], {}),
            'votes': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        'djangoratings.vote': {
            'Meta': {'unique_together': "(('content_type', 'object_id', 'key', 'user', 'ip_address'),)"},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'votes'", 'to': "orm['contenttypes.ContentType']"}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_changed': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_address': ('django.db.models.fields.IPAddressField', [], {'max_length': '15'}),
            'key': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'score': ('django.db.models.fields.IntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'votes'", 'null': 'True', 'to': "orm['auth.User']"})
        }
    }
    
    complete_apps = ['djangoratings']



from django.db.models import Manager
from django.db.models.query import QuerySet

from django.contrib.contenttypes.models import ContentType
import itertools

class VoteQuerySet(QuerySet):
    def delete(self, *args, **kwargs):
        """Handles updating the related `votes` and `score` fields attached to the model."""
        # XXX: circular import
        from fields import RatingField

        qs = self.distinct().values_list('content_type', 'object_id').order_by('content_type')
    
        to_update = []
        for content_type, objects in itertools.groupby(qs, key=lambda x: x[0]):
            model_class = ContentType.objects.get(pk=content_type).model_class()
            if model_class:
                to_update.extend(list(model_class.objects.filter(pk__in=list(objects)[0])))
        
        retval = super(VoteQuerySet, self).delete(*args, **kwargs)
        
        # TODO: this could be improved
        for obj in to_update:
            for field in getattr(obj, '_djangoratings', []):
                getattr(obj, field.name)._update(commit=False)
            obj.save()
        
        return retval
        
class VoteManager(Manager):
    def get_query_set(self):
        return VoteQuerySet(self.model)

    def get_for_user_in_bulk(self, objects, user):
        objects = list(objects)
        if len(objects) > 0:
            ctype = ContentType.objects.get_for_model(objects[0])
            votes = list(self.filter(content_type__pk=ctype.id,
                                     object_id__in=[obj._get_pk_val() \
                                                    for obj in objects],
                                     user__pk=user.id))
            vote_dict = dict([(vote.object_id, vote) for vote in votes])
        else:
            vote_dict = {}
        return vote_dict

class SimilarUserManager(Manager):
    def get_recommendations(self, user, model_class, min_score=1):
        from djangoratings.models import Vote, IgnoredObject
        
        content_type = ContentType.objects.get_for_model(model_class)
        
        params = dict(
            v=Vote._meta.db_table,
            sm=self.model._meta.db_table,
            m=model_class._meta.db_table,
            io=IgnoredObject._meta.db_table,
        )
        
        objects = model_class._default_manager.extra(
            tables=[params['v']],
            where=[
                '%(v)s.object_id = %(m)s.id and %(v)s.content_type_id = %%s' % params,
                '%(v)s.user_id IN (select to_user_id from %(sm)s where from_user_id = %%s and exclude = 0)' % params,
                '%(v)s.score >= %%s' % params,
                # Exclude already rated maps
                '%(v)s.object_id NOT IN (select object_id from %(v)s where content_type_id = %(v)s.content_type_id and user_id = %%s)' % params,
                # IgnoredObject exclusions
                '%(v)s.object_id NOT IN (select object_id from %(io)s where content_type_id = %(v)s.content_type_id and user_id = %%s)' % params,
            ],
            params=[content_type.id, user.id, min_score, user.id, user.id]
        ).distinct()

        # objects = model_class._default_manager.filter(pk__in=content_type.votes.extra(
        #     where=['user_id IN (select to_user_id from %s where from_user_id = %d and exclude = 0)' % (self.model._meta.db_table, user.pk)],
        # ).filter(score__gte=min_score).exclude(
        #     object_id__in=IgnoredObject.objects.filter(content_type=content_type, user=user).values_list('object_id', flat=True),
        # ).exclude(
        #     object_id__in=Vote.objects.filter(content_type=content_type, user=user).values_list('object_id', flat=True)
        # ).distinct().values_list('object_id', flat=True))
        
        return objects
    
    def update_recommendations(self):
        # TODO: this is mysql only atm
        # TODO: this doesnt handle scores that have multiple values (e.g. 10 points, 5 stars)
        # due to it calling an agreement as score = score. We need to loop each rating instance
        # and express the condition based on the range.
        from djangoratings.models import Vote
        from django.db import connection
        cursor = connection.cursor()
        cursor.execute('begin')
        cursor.execute('truncate table %s' % (self.model._meta.db_table,))
        cursor.execute("""insert into %(t1)s
          (to_user_id, from_user_id, agrees, disagrees, exclude)
          select v1.user_id, v2.user_id,
                 sum(if(v2.score = v1.score, 1, 0)) as agrees,
                 sum(if(v2.score != v1.score, 1, 0)) as disagrees, 0
            from %(t2)s as v1
              inner join %(t2)s as v2
                on v1.user_id != v2.user_id
                and v1.object_id = v2.object_id
                and v1.content_type_id = v2.content_type_id
            where v1.user_id is not null
              and v2.user_id is not null
            group by v1.user_id, v2.user_id
            having agrees / (disagrees + 0.0001) > 3
          on duplicate key update agrees = values(agrees), disagrees = values(disagrees);""" % dict(
            t1=self.model._meta.db_table,
            t2=Vote._meta.db_table,
        ))
        cursor.execute('commit')
        cursor.close()


import os.path
import warnings

__version__ = (0, 3, 7)

def _get_git_revision(path):
    revision_file = os.path.join(path, 'refs', 'heads', 'master')
    if not os.path.exists(revision_file):
        return None
    fh = open(revision_file, 'r')
    try:
        return fh.read()
    finally:
        fh.close()

def get_revision():
    """
    :returns: Revision number of this branch/checkout, if available. None if
        no revision number can be determined.
    """
    package_dir = os.path.dirname(__file__)
    checkout_dir = os.path.normpath(os.path.join(package_dir, '..'))
    path = os.path.join(checkout_dir, '.git')
    if os.path.exists(path):
        return _get_git_revision(path)
    return None

__build__ = get_revision()

def lazy_object(location):
    def inner(*args, **kwargs):
        parts = location.rsplit('.', 1)
        warnings.warn('`djangoratings.%s` is deprecated. Please use `%s` instead.' % (parts[1], location), DeprecationWarning)
        try:
            imp = __import__(parts[0], globals(), locals(), [parts[1]], -1)
        except:
            imp = __import__(parts[0], globals(), locals(), [parts[1]])
        func = getattr(imp, parts[1])
        if callable(func):
            return func(*args, **kwargs)
        return func
    return inner

RatingField = lazy_object('djangoratings.fields.RatingField')
AnonymousRatingField = lazy_object('djangoratings.fields.AnonymousRatingField')
Rating = lazy_object('djangoratings.fields.Rating')


from django.conf import settings

# Used to limit the number of unique IPs that can vote on a single object+field.
#   useful if you're getting rating spam by users registering multiple accounts
RATINGS_VOTES_PER_IP = 3


#!/usr/bin/env python

from setuptools import setup, find_packages

tests_require = [
    'django',
    # also requires the disqus fork of haystack
]

setup(
    name='django-ratings',
    version=".".join(map(str, __import__('djangoratings').__version__)),
    author='David Cramer',
    author_email='dcramer@gmail.com',
    description='Generic Ratings in Django',
    url='http://github.com/dcramer/django-ratings',
    install_requires=[
        'django',
    ],
    tests_require=tests_require,
    extras_require={'test': tests_require},
    test_suite='djangoratings.runtests.runtests',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development"
    ],
)